@@ -9,70 +9,56 @@
     :license: MIT License
 """
 
-import re
-import json
+from json import loads as jloads
+from re import compile as rcompile
 
 from nektar import Waggle
 from emoji import emoji_list
 from langdetect import detect_langs
 
-RE_DASH = re.compile(r"-")
-RE_EN_DASH = re.compile(r"\u2013")
-RE_EM_DASH = re.compile(r"\u2014")
-RE_N_RANK = re.compile(r"\#[\d]+")
-RE_DOLLARS = re.compile(r"\$[\d\,\.]+")
-RE_POSSESSIVE = re.compile(r"\b[\w]+[']s\b")
-RE_MULTI_SPACE = re.compile(r"\s{2,}")
-RE_UPPERCASE = re.compile(r"[A-Z]")
-RE_CLEAN_TITLE = re.compile(r"[^\w\'\,\-\ ]+")
-RE_DELIMITERS = re.compile(r"[\n\.]")
-RE_IMAGE = re.compile(r"!\[[^\]]*\]\([^\)]+\)")
-RE_IMAGES = re.compile(r"!\[[^\]]*\]\([^\)]+\)[\s]{0,}!\[[^\]]*\]\([^\)]+\)")
-RE_HIVE_SVC = re.compile(r"\[\/\/\]:#[\ ]+\([!][\w\ \.]+\)")
-RE_ASTERISKS = re.compile(r"[\*]+")
-RE_TILDES = re.compile(r"[~]+")
-RE_UNDERSCORES = re.compile(r"[_]+")
-RE_HEADERS = re.compile(r"[#][#]+")
-RE_CODE_BLOCKS = re.compile(r"[`]+[\w]*[\ ]*")
-RE_TABLE_SEP = re.compile(r"\|[\-]+\|[\-]+\|")
-RE_PIPES = re.compile(r"[\ ]*\|[\ ]*")
-RE_HTML_TAGS = re.compile(r"<[\/]?[a-zA-Z]+[1-6]?[^\>]+>")
-RE_LINKS_RIGHT = re.compile(r"\]\([^\)]+\)")
-RE_BLOCKQUOTES = re.compile(r">[\ ]?")
-RE_HR = re.compile(r"--[\-]+")
-RE_TRAILING_PARENTHESIS = re.compile(r"[\(\[\{\}\]\)]")
-RE_USER_TAGS = re.compile(r"[^\w\/]@[\w\-\.]{3,16}[^\w\/]")
-RE_NON_ASCII = re.compile(r"[^ -~]")
+RE_DASH = rcompile(r"(\-|\u2013|\u2014)")
+RE_N_RANK = rcompile(r"\#[\d]+")
+RE_DOLLARS = rcompile(r"\$[\d\,\.]+")
+RE_POSSESSIVE = rcompile(r"\b[\w]+[']s\b")
+RE_MULTI_SPACE = rcompile(r"\s{2,}")
+RE_UPPERCASE = rcompile(r"[A-Z]")
+RE_CLEAN_TITLE = rcompile(r"[^\w\'\,\-\ ]+")
+RE_DELIMITERS = rcompile(r"[\n\.]")
+RE_IMAGE = rcompile(r"!\[[^\]]*\]\([^\)]+\)")
+RE_IMAGES = rcompile(r"!\[[^\]]*\]\([^\)]+\)\s*!\[[^\]]*\]\([^\)]+\)")
+RE_HIVE_SVC = rcompile(r"\[\/\/\]:#[\ ]+\([!][\w\ \.]+\)")
+RE_ASTERISKS = rcompile(r"[\*]+")
+RE_TILDES = rcompile(r"[~]+")
+RE_UNDERSCORES = rcompile(r"[_]+")
+RE_HEADERS = rcompile(r"[#][#]+")
+RE_CODE_BLOCKS = rcompile(r"[`]+[\w]*[\ ]*")
+RE_TABLE_SEP = rcompile(r"\|[\-:]+\|[\-:]+\|")
+RE_PIPES = rcompile(r"[\ :]*\|[\ :]*")
+RE_HTML_TAGS = rcompile(r"<[\/]?[a-zA-Z]+[1-6]?[^\>]+>")
+RE_LINKS_RIGHT = rcompile(r"\]\([^\)]+\)")
+RE_BLOCKQUOTES = rcompile(r">[\ ]?")
+RE_HR = rcompile(r"--[\-]+")
+RE_TRAILING_PARENTHESIS = rcompile(r"[\(\[\{\}\]\)]")
+RE_USER_TAGS = rcompile(r"[^\w\/]@[\w\-\.]{3,16}[^\w\/]")
+RE_NUMBERS = rcompile(r"\d+\.?\d*")
+RE_PUNCTUATIONS = rcompile(r"[\.\,\!\?]")
+RE_NON_ASCII = rcompile(r"[^ -~]")
+RE_WORD = rcompile(r"\w[^\s]+")
 
 STOP_WORDS = [
     "0s",
-    "3a",
-    "3b",
-    "3d",
-    "6b",
-    "6o",
     "a",
-    "a's",
-    "a1",
-    "a2",
-    "a3",
-    "a4",
-    "ab",
     "able",
     "about",
     "above",
-    "abst",
-    "ac",
     "accordance",
     "according",
     "accordingly",
     "across",
     "act",
     "actually",
-    "ad",
     "added",
     "adj",
-    "ae",
     "af",
     "affected",
     "affecting",
@@ -82,11 +68,7 @@ STOP_WORDS = [
     "ag",
     "again",
     "against",
-    "ah",
-    "ain",
     "ain't",
-    "aj",
-    "al",
     "all",
     "allow",
     "allows",
@@ -115,15 +97,12 @@ STOP_WORDS = [
     "anyway",
     "anyways",
     "anywhere",
-    "ao",
-    "ap",
     "apart",
     "apparently",
     "appear",
     "appreciate",
     "appropriate",
     "approximately",
-    "ar",
     "are",
     "aren",
     "aren't",
@@ -136,24 +115,11 @@ STOP_WORDS = [
     "asking",
     "associated",
     "at",
-    "au",
-    "auth",
-    "av",
     "available",
     "aw",
     "away",
     "awfully",
-    "ax",
-    "ay",
-    "az",
-    "b",
-    "b1",
-    "b2",
-    "b3",
-    "ba",
     "back",
-    "bc",
-    "bd",
     "be",
     "became",
     "because",
@@ -177,32 +143,15 @@ STOP_WORDS = [
     "better",
     "between",
     "beyond",
-    "bi",
     "bill",
-    "biol",
-    "bj",
-    "bk",
-    "bl",
-    "bn",
     "both",
     "bottom",
-    "bp",
-    "br",
     "brief",
     "briefly",
     "bs",
-    "bt",
-    "bu",
     "but",
     "bx",
-    "by",
-    "c",
     "c'mon",
-    "c's",
-    "c1",
-    "c2",
-    "c3",
-    "ca",
     "call",
     "came",
     "can",
@@ -211,24 +160,11 @@ STOP_WORDS = [
     "cant",
     "cause",
     "causes",
-    "cc",
-    "cd",
-    "ce",
     "certain",
     "certainly",
-    "cf",
-    "cg",
-    "ch",
     "changes",
-    "ci",
-    "cit",
-    "cj",
-    "cl",
     "clearly",
-    "cm",
-    "cn",
     "co",
-    "com",
     "come",
     "comes",
     "con",
@@ -245,38 +181,17 @@ STOP_WORDS = [
     "couldn't",
     "couldnt",
     "course",
-    "cp",
-    "cq",
-    "cr",
     "cry",
-    "cs",
-    "ct",
-    "cu",
     "currently",
-    "cv",
-    "cx",
-    "cy",
-    "cz",
-    "d",
-    "d2",
-    "da",
     "date",
-    "dc",
-    "dd",
     "definitely",
     "describe",
     "described",
     "despite",
     "detail",
-    "df",
-    "di",
     "did",
-    "didn",
     "didn't",
     "different",
-    "dj",
-    "dk",
-    "dl",
     "do",
     "does",
     "doesn",
@@ -287,53 +202,27 @@ STOP_WORDS = [
     "done",
     "down",
     "downwards",
-    "dp",
     "dr",
-    "ds",
-    "dt",
-    "du",
     "due",
     "during",
-    "dx",
-    "dy",
-    "e",
-    "e2",
-    "e3",
-    "ea",
     "each",
-    "ec",
-    "ed",
-    "edu",
-    "ee",
-    "ef",
     "effect",
     "eg",
-    "ei",
     "eight",
     "eighty",
     "either",
-    "ej",
     "eleven",
     "else",
     "elsewhere",
-    "em",
     "empty",
     "end",
     "ending",
     "enough",
     "entirely",
-    "eo",
-    "ep",
-    "eq",
-    "er",
-    "es",
     "especially",
-    "est",
     "et",
     "et-al",
     "etc",
-    "eu",
-    "ev",
     "even",
     "ever",
     "every",
@@ -341,19 +230,12 @@ STOP_WORDS = [
     "everyone",
     "everything",
     "everywhere",
-    "ex",
     "exactly",
     "example",
     "except",
-    "ey",
-    "f",
-    "f2",
-    "fa",
     "far",
-    "fc",
     "few",
     "ff",
-    "fi",
     "fifteen",
     "fifth",
     "fify",
@@ -363,10 +245,6 @@ STOP_WORDS = [
     "first",
     "five",
     "fix",
-    "fj",
-    "fl",
-    "fn",
-    "fo",
     "followed",
     "following",
     "follows",
@@ -380,39 +258,27 @@ STOP_WORDS = [
     "fr",
     "from",
     "front",
-    "fs",
     "ft",
-    "fu",
     "full",
     "further",
     "furthermore",
-    "fy",
-    "g",
-    "ga",
     "gave",
     "ge",
     "get",
     "gets",
     "getting",
-    "gi",
     "give",
     "giveaway",
     "given",
     "gives",
     "giving",
-    "gj",
-    "gl",
     "go",
     "goes",
     "going",
     "gone",
     "got",
     "gotten",
-    "gr",
     "greetings",
-    "gs",
-    "gy",
-    "h",
     "h2",
     "h3",
     "had",
@@ -432,7 +298,6 @@ STOP_WORDS = [
     "he'd",
     "he'll",
     "he's",
-    "hed",
     "hello",
     "help",
     "hence",
@@ -447,14 +312,12 @@ STOP_WORDS = [
     "hers",
     "herself",
     "hes",
-    "hh",
     "hi",
     "hid",
     "him",
     "himself",
     "his",
     "hither",
-    "hj",
     "ho",
     "home",
     "hopefully",
@@ -462,37 +325,16 @@ STOP_WORDS = [
     "how's",
     "howbeit",
     "however",
-    "hr",
-    "hs",
     "http",
-    "hu",
     "hundred",
-    "hy",
     "i",
     "i'd",
     "i'll",
     "i'm",
     "i've",
-    "i2",
-    "i3",
-    "i4",
-    "i6",
-    "i7",
-    "i8",
-    "ia",
-    "ib",
-    "ibid",
-    "ic",
-    "id",
     "ie",
     "if",
-    "ig",
     "ignored",
-    "ih",
-    "ii",
-    "ij",
-    "il",
-    "im",
     "immediate",
     "immediately",
     "importance",
@@ -514,54 +356,29 @@ STOP_WORDS = [
     "invention",
     "inward",
     "io",
-    "ip",
-    "iq",
-    "ir",
     "is",
-    "isn",
     "isn't",
     "it",
     "it'd",
     "it'll",
     "it's",
-    "itd",
     "its",
     "itself",
-    "iv",
-    "ix",
-    "iy",
-    "iz",
-    "j",
-    "jj",
-    "jr",
-    "js",
-    "jt",
-    "ju",
     "just",
-    "k",
-    "ke",
     "keep",
     "keeps",
     "kept",
     "kg",
-    "kj",
     "km",
     "know",
     "known",
     "knows",
-    "ko",
-    "l",
-    "l2",
-    "la",
     "largely",
     "last",
     "lately",
     "later",
     "latter",
     "latterly",
-    "lb",
-    "lc",
-    "le",
     "least",
     "les",
     "less",
@@ -575,21 +392,10 @@ STOP_WORDS = [
     "likely",
     "line",
     "little",
-    "lj",
-    "ll",
-    "ln",
-    "lo",
     "look",
     "looking",
     "looks",
-    "los",
-    "lr",
-    "ls",
-    "lt",
     "ltd",
-    "m",
-    "m2",
-    "ma",
     "made",
     "mainly",
     "make",
@@ -622,8 +428,6 @@ STOP_WORDS = [
     "mr",
     "mrs",
     "ms",
-    "mt",
-    "mu",
     "much",
     "mug",
     "must",
@@ -631,7 +435,6 @@ STOP_WORDS = [
     "my",
     "myself",
     "n",
-    "n2",
     "na",
     "name",
     "namely",
@@ -653,12 +456,8 @@ STOP_WORDS = [
     "new",
     "next",
     "ng",
-    "ni",
     "nine",
     "ninety",
-    "nj",
-    "nl",
-    "nn",
     "no",
     "nobody",
     "non",
@@ -674,13 +473,6 @@ STOP_WORDS = [
     "novel",
     "now",
     "nowhere",
-    "nr",
-    "ns",
-    "nt",
-    "ny",
-    "o",
-    "oa",
-    "ob",
     "obtain",
     "obtained",
     "obviously",
@@ -689,15 +481,10 @@ STOP_WORDS = [
     "of",
     "off",
     "often",
-    "og",
     "oh",
-    "oi",
-    "oj",
     "ok",
     "okay",
-    "ol",
     "old",
-    "om",
     "omitted",
     "on",
     "once",
@@ -705,17 +492,11 @@ STOP_WORDS = [
     "ones",
     "only",
     "onto",
-    "oo",
     "op",
-    "oq",
     "or",
-    "ord",
-    "os",
-    "ot",
     "other",
     "others",
     "otherwise",
-    "ou",
     "ought",
     "our",
     "ours",
@@ -724,12 +505,8 @@ STOP_WORDS = [
     "outside",
     "over",
     "overall",
-    "ow",
     "owing",
     "own",
-    "ox",
-    "oz",
-    "p",
     "p1",
     "p2",
     "p3",
@@ -742,30 +519,15 @@ STOP_WORDS = [
     "particularly",
     "pas",
     "past",
-    "pc",
-    "pd",
-    "pe",
     "per",
     "perhaps",
-    "pf",
-    "ph",
-    "pi",
-    "pj",
-    "pk",
-    "pl",
     "placed",
     "please",
     "plus",
-    "pm",
-    "pn",
-    "po",
     "poorly",
     "possible",
     "possibly",
     "potentially",
-    "pp",
-    "pq",
-    "pr",
     "predominantly",
     "present",
     "presumably",
@@ -776,24 +538,12 @@ STOP_WORDS = [
     "proud",
     "provides",
     "ps",
-    "pt",
-    "pu",
     "put",
-    "py",
-    "q",
-    "qj",
-    "qu",
     "que",
     "quickly",
     "quite",
-    "qv",
-    "r",
-    "r2",
-    "ra",
     "ran",
     "rather",
-    "rc",
-    "rd",
     "re",
     "readily",
     "really",
@@ -807,41 +557,18 @@ STOP_WORDS = [
     "regards",
     "related",
     "relatively",
-    "research",
-    "research-articl",
     "respectively",
     "resulted",
     "resulting",
     "results",
-    "rf",
-    "rh",
-    "ri",
     "right",
-    "rj",
-    "rl",
-    "rm",
-    "rn",
-    "ro",
-    "rq",
-    "rr",
-    "rs",
-    "rt",
-    "ru",
     "run",
-    "rv",
-    "ry",
-    "s",
-    "s2",
-    "sa",
     "said",
     "same",
     "saw",
     "say",
     "saying",
     "says",
-    "sc",
-    "sd",
-    "se",
     "sec",
     "second",
     "secondly",
@@ -861,10 +588,7 @@ STOP_WORDS = [
     "seriously",
     "seven",
     "several",
-    "sf",
     "shall",
-    "shan",
-    "shan't",
     "she",
     "she'd",
     "she'll",
@@ -880,7 +604,6 @@ STOP_WORDS = [
     "shown",
     "showns",
     "shows",
-    "si",
     "side",
     "significant",
     "significantly",
@@ -890,11 +613,7 @@ STOP_WORDS = [
     "sincere",
     "six",
     "sixty",
-    "sj",
-    "sl",
     "slightly",
-    "sm",
-    "sn",
     "so",
     "some",
     "somebody",
@@ -908,15 +627,10 @@ STOP_WORDS = [
     "somewhere",
     "soon",
     "sorry",
-    "sp",
     "specifically",
     "specified",
     "specify",
     "specifying",
-    "sq",
-    "sr",
-    "ss",
-    "st",
     "still",
     "stop",
     "strongly",
@@ -928,26 +642,13 @@ STOP_WORDS = [
     "suggest",
     "sup",
     "sure",
-    "sy",
     "system",
-    "sz",
-    "t",
-    "t's",
-    "t1",
-    "t2",
-    "t3",
     "take",
     "taken",
     "taking",
-    "tb",
-    "tc",
-    "td",
-    "te",
     "tell",
     "ten",
     "tends",
-    "tf",
-    "th",
     "than",
     "thank",
     "thanks",
@@ -986,7 +687,7 @@ STOP_WORDS = [
     "they've",
     "theyd",
     "theyre",
-    "thickv",
+    "thick",
     "thin",
     "think",
     "third",
@@ -1004,13 +705,8 @@ STOP_WORDS = [
     "throughout",
     "thru",
     "thus",
-    "ti",
     "til",
     "tip",
-    "tj",
-    "tl",
-    "tm",
-    "tn",
     "to",
     "together",
     "too",
@@ -1018,29 +714,15 @@ STOP_WORDS = [
     "top",
     "toward",
     "towards",
-    "tp",
-    "tq",
-    "tr",
     "tried",
     "tries",
     "truly",
     "try",
     "trying",
-    "ts",
-    "tt",
-    "tv",
     "twelve",
     "twenty",
     "twice",
     "two",
-    "tx",
-    "u",
-    "u201d",
-    "ue",
-    "ui",
-    "uj",
-    "uk",
-    "um",
     "un",
     "under",
     "unfortunately",
@@ -1049,11 +731,9 @@ STOP_WORDS = [
     "unlikely",
     "until",
     "unto",
-    "uo",
     "up",
     "upon",
     "ups",
-    "ur",
     "us",
     "use",
     "used",
@@ -1063,27 +743,13 @@ STOP_WORDS = [
     "uses",
     "using",
     "usually",
-    "ut",
-    "v",
-    "va",
     "value",
     "various",
-    "vd",
-    "ve",
     "very",
     "via",
     "viz",
-    "vj",
-    "vo",
     "vol",
     "vols",
-    "volumtype",
-    "vq",
-    "vs",
-    "vt",
-    "vu",
-    "w",
-    "wa",
     "want",
     "wants",
     "was",
@@ -1099,7 +765,6 @@ STOP_WORDS = [
     "wed",
     "welcome",
     "well",
-    "well-b",
     "went",
     "were",
     "weren",
@@ -1136,11 +801,9 @@ STOP_WORDS = [
     "whole",
     "whom",
     "whomever",
-    "whos",
     "whose",
     "why",
     "why's",
-    "wi",
     "widely",
     "will",
     "willing",
@@ -1148,8 +811,6 @@ STOP_WORDS = [
     "with",
     "within",
     "without",
-    "wo",
-    "won",
     "won't",
     "wonder",
     "wont",
@@ -1158,50 +819,22 @@ STOP_WORDS = [
     "would",
     "wouldn",
     "wouldn't",
-    "wouldnt",
     "www",
-    "x",
-    "x1",
-    "x2",
-    "x3",
-    "xf",
-    "xi",
-    "xj",
-    "xk",
-    "xl",
-    "xn",
-    "xo",
-    "xs",
-    "xt",
-    "xv",
-    "xx",
-    "y",
-    "y2",
     "yes",
     "yet",
-    "yj",
-    "yl",
     "you",
     "you'd",
     "you'll",
     "you're",
     "you've",
-    "youd",
     "your",
-    "youre",
+    "you're",
     "yours",
     "yourself",
     "yourselves",
-    "yr",
-    "ys",
-    "yt",
-    "z",
     "zero",
-    "zi",
-    "zz",
 ]
 
-
 class Scrutineer:
     def __init__(
         self,
@@ -1221,9 +854,11 @@ class Scrutineer:
         self._retries = int(retries)
         self._deep = isinstance(deep, bool) * bool(deep)
         self._full = isinstance(full, bool) * bool(full)
-        self.analysis = {}
-        self._blogs = []
-        self._previous = ""
+        self._permlink = None
+        self._previous = None
+        self._template = []
+        self._analysis = {}
+        self._waggle = Waggle("")
 
     def set_weights(self, title=1, body=1, emojis=1, images=1, tagging=1, tags=1):
         self._weights = [
@@ -1236,104 +871,99 @@ class Scrutineer:
         ]
 
     def analyze(self, post, permlink=None, auto_skip=False):
-        author = post
+        self._analysis = {}
+        
         if isinstance(post, dict):
             author = post["author"]
             permlink = post["permlink"]
-        account = Waggle(author)
         if not isinstance(post, dict):
-            post = account.get_post(author, permlink, retries=self._retries)
+            author = post
+            post = self._waggle.get_post(author, permlink, retries=self._retries)
             if not post:
                 return {}
 
-        self.analysis["author"] = author
-        self.analysis["permlink"] = permlink
+        self._analysis["author"] = author
+        self._analysis["permlink"] = permlink
 
         if self._full:
-            self.analysis["url"] = post["url"]
+            self._analysis["url"] = post["url"]
 
         title = post["title"]
         if not len(title):
             return {}
-        self.analysis["title"] = {}
 
         body = post["body"]
         if self._deep:
-            unique_lines = []
             raw_body = body.split("\n")
-            if author == self._previous:
-                blogs = self._blogs
-            else:
+            if author != self._previous or (permlink == self._permlink and author == self._previous):
                 self._previous = author
-                blogs = account.blogs(author, limit=2)
-                self._blogs = [b for b in blogs]
-            for blog in blogs:
-                if blog["permlink"] == permlink:
-                    continue
-                raw_blog_body = blog["body"].split("\n")
-                unique_lines = [l for l in raw_body if l not in raw_blog_body]
-                break
-            body = "\n".join(unique_lines)
+                for blog in self._waggle.blogs(author, limit=2):
+                    if blog["permlink"] == self._permlink:
+                        continue
+                    self._permlink = blog["permlink"]
+                    self._template = blog["body"].split("\n")
+                    break
+            body = "\n".join([l for l in raw_body
+                if l not in self._template])
         cleaned = _parse_body(body)
         if not len(cleaned):
             return {}
 
-        keywords = _get_bigrams(cleaned)
-        self.analysis["title"] = _analyze_title(title, keywords, self._full)
+        # use keywords instead
+        keywords = get_keywords(cleaned) # _get_bigrams(cleaned)
+        self._analysis["title"] = _analyze_title(title, keywords, self._full)
 
-        self.analysis["body"] = {}
-        self.analysis["emojis"] = _analyze_emojis(body, self._max_emojis, self._full)
+        self._analysis["body"] = {}
+        self._analysis["emojis"] = _analyze_emojis(body, self._max_emojis, self._full)
         if auto_skip:
             if self._full:
                 if (
-                    self.analysis["emojis"]["score"] < 0.8
-                    or self.analysis["title"]["score"] < 0.8
+                    self._analysis["emojis"]["score"] < 0.8
+                    or self._analysis["title"]["score"] < 0.8
                 ):
                     return {}
-            elif self.analysis["emojis"] < 0.8 or self.analysis["title"] < 0.8:
+            elif self._analysis["emojis"] < 0.8 or self._analysis["title"] < 0.8:
                 return {}
-        self.analysis["body"] = _analyze_body(cleaned, self._deep, self._full)
+        self._analysis["body"] = _analyze_body(cleaned, self._deep, self._full)
 
         wcount = len(cleaned.split(" "))
-        self.analysis["images"] = _analyze_images(body, wcount, self._full)
-        self.analysis["tagging"] = _analyze_overtagging(
+        self._analysis["images"] = _analyze_images(body, wcount, self._full)
+        self._analysis["tagging"] = _analyze_overtagging(
             body, self._max_user_tags, self._full
         )
 
         metadata = post["json_metadata"]
         if isinstance(metadata, str):
-            metadata = json.loads(metadata)
+            metadata = jloads(metadata)
         tags = metadata.get("tags", [])
-        self.analysis["tags"] = _analyze_tags(tags, self._max_tags, self._full)
+        self._analysis["tags"] = _analyze_tags(tags, self._max_tags, self._full)
 
         score = 0
         if not self._full:
-            score += self.analysis["title"] * self._weights[0]
-            score += self.analysis["body"] * self._weights[1]
-            score += self.analysis["emojis"] * self._weights[2]
-            score += self.analysis["images"] * self._weights[3]
-            score += self.analysis["tagging"] * self._weights[4]
-            score += self.analysis["tags"] * self._weights[5]
+            score += self._analysis["title"] * self._weights[0]
+            score += self._analysis["body"] * self._weights[1]
+            score += self._analysis["emojis"] * self._weights[2]
+            score += self._analysis["images"] * self._weights[3]
+            score += self._analysis["tagging"] * self._weights[4]
+            score += self._analysis["tags"] * self._weights[5]
         else:
-            score += self.analysis["title"]["score"] * self._weights[0]
-            score += self.analysis["body"]["score"] * self._weights[1]
-            score += self.analysis["emojis"]["score"] * self._weights[2]
-            score += self.analysis["images"]["score"] * self._weights[3]
-            score += self.analysis["tagging"]["score"] * self._weights[4]
-            score += self.analysis["tags"]["score"] * self._weights[5]
+            score += self._analysis["title"]["score"] * self._weights[0]
+            score += self._analysis["body"]["score"] * self._weights[1]
+            score += self._analysis["emojis"]["score"] * self._weights[2]
+            score += self._analysis["images"]["score"] * self._weights[3]
+            score += self._analysis["tagging"]["score"] * self._weights[4]
+            score += self._analysis["tags"]["score"] * self._weights[5]
         score /= sum(self._weights)
 
-        self.analysis["deep"] = self._deep
-        self.analysis["score"] = score
-        return self.analysis
+        self._analysis["deep"] = self._deep
+        self._analysis["score"] = score
+        return self._analysis
 
 
 def _analyze_title(title, keywords, full=False):
 
     cleaned = title
     cleaned = RE_DASH.sub(" ", cleaned)
-    cleaned = RE_EN_DASH.sub(" ", cleaned)
-    cleaned = RE_EM_DASH.sub(" ", cleaned)
     cleaned = RE_N_RANK.sub(" ", cleaned)
     cleaned = RE_DOLLARS.sub(" ", cleaned)
     cleaned = RE_POSSESSIVE.sub("", cleaned)
@@ -1343,7 +973,6 @@ def _analyze_title(title, keywords, full=False):
 
     bmin = length < 20
     amax = length > 80
-    
 
     score = 0
     skeywords = 0
@@ -1353,17 +982,17 @@ def _analyze_title(title, keywords, full=False):
     if length and not len(emojis):
         uppercase = len(RE_UPPERCASE.findall(cleaned))/length
         adjust = (1, 0.5)[int(bool(uppercase>0.5))]
-        english = _to_english(cleaned, chars=True)
+        
+        english = _count_english(cleaned, chars=True)
         readability = (english / len(title)) * adjust
+        
         if isinstance(keywords, dict):
             words = title.lower()
-            for keyword in keywords.keys():
+            for keyword in keywords:
                 if keyword in words:
-                    skeywords = 1
+                    skeywords = 0.5
                     break
-        score = int(not (bmin or amax)) * (
-            ((readability * 9.5) + (skeywords * 0.5)) / 10
-        )
+        score = int(not (bmin or amax)) * (((readability * 9.5) + skeywords) / 10)
 
     if not full:
         return score
@@ -1381,34 +1010,17 @@ def _analyze_title(title, keywords, full=False):
     }
 
 def get_keywords(body, occurrence=4):
-    occurrence = int(occurrence)
-    if not body:
-        return {}
-     
-    keywords = {}
-    cleaned = _parse_body(body)
-    words = cleaned.lower().split(" ")
-    words = [w for w in words if w not in STOP_WORDS]
-    for word in set(words):
-        count = words.count(word)
-        if count < occurrence:
-            continue
-        keywords[word] = count
-    return keywords
+    words = RE_WORD.findall(_parse_body(body).lower())
+    keywords = {w:words.count(w) for w in set(words) if w not in STOP_WORDS}
+    return {k: c for k, c in keywords.items() if c >= int(occurrence)}
 
 def get_bigrams(body, occurrence=4):
-    occurrence = int(occurrence)
-    if not body:
-        return {}
-    cleaned = _parse_body(body)
-    return _get_bigrams(cleaned, occurrence=4)
+    return _get_bigrams(_parse_body(body), occurrence=int(occurrence))
 
 def _parse_body(body):
     # remove images, replace whitespaces
     cleaned = RE_IMAGE.sub("", body).lower()
     cleaned = RE_DELIMITERS.sub(" ", cleaned)
-    cleaned = RE_EN_DASH.sub(" ", cleaned)
-    cleaned = RE_EM_DASH.sub(" ", cleaned)
 
     ## remove other formatting codes
     ## do not change order of patterns !!
@@ -1425,30 +1037,30 @@ def _parse_body(body):
     cleaned = RE_BLOCKQUOTES.sub("", cleaned)
     cleaned = RE_HR.sub("", cleaned)
     cleaned = RE_TRAILING_PARENTHESIS.sub("", cleaned)
-    cleaned = RE_USER_TAGS.sub("", cleaned)
+    cleaned = RE_USER_TAGS.sub(" ", cleaned)
     cleaned = RE_DASH.sub(" ", cleaned)
+    cleaned = RE_PUNCTUATIONS.sub(" ", cleaned)
+    cleaned = RE_NUMBERS.sub(" ", cleaned)
     cleaned = RE_MULTI_SPACE.sub(" ", cleaned)
 
     return cleaned
 
 def _get_bigrams(contents, occurrence=4):
-    occurrence = int(occurrence)
-
     bigrams = {}
-    words = contents.lower().split(" ")
+    words = RE_WORD.findall(contents.lower())
     words = [w for w in words if w not in STOP_WORDS]
     for i in range(len(words) - 2):
         bigram = " ".join(words[i : i + 2])
         bigrams[bigram] = bigrams.get(bigram, 0) + 1
-    return {b: o for b, o in bigrams.items() if o >= occurrence}
+    return {b: o for b, o in bigrams.items() if o >= int(occurrence)}
 
 
 def _analyze_body(words, deep, full=False):
     length = len(words.split(" "))
-    english = _to_english(words)
+    english = _count_english(words)
     w400 = english > 400
     w800 = english > 800
-    score = ((english / length) + (w400 * 2) + w800) / 4
+    score = (w400 + w800) * (english / length) / 2
 
     if not full:
         return score
@@ -1461,31 +1073,28 @@ def _analyze_body(words, deep, full=False):
     }
 
 
-def _to_english(words, chars=False):
-    if not len(words):
+def _count_english(text, chars=False):
+    if not len(text):
         return 0
     try:
-        results = str(detect_langs(words))[1:-1].split(",")
+        results = str(detect_langs(text))[1:-1].split(",")
     except Exception as e:
         print(f"Scrutineer: {e}")
         return 0
     for lang in results:
-        if "en:" in lang:
-            if chars:
-                return float(lang.strip()[3:]) * len(words)
-            return float(lang.strip()[3:]) * len(words.split(" "))
+        if "en:" not in lang:
+            continue
+        if chars:
+            return float(lang.strip()[3:]) * len(text)
+        return float(lang.strip()[3:]) * len(text.split(" "))
     return 0
 
-
 def _analyze_emojis(body, limit, full=False):
-    emojis = emoji_list(body)
-
     score = 1
+    emojis = emoji_list(body)
     count = len(emojis)
     if count > limit:
-        score = 0
-        if limit:
-            score = limit / count
+        score = (limit / count) * int(bool(limit))
 
     if not full:
         return score
@@ -1499,8 +1108,8 @@ def _analyze_emojis(body, limit, full=False):
 
 def _analyze_images(body, wcount, full=True):
     score = 0
-    count = len(list(RE_IMAGE.findall(body)))
-    sequences = len(list(RE_IMAGES.findall(body)))
+    count = len(RE_IMAGE.findall(body))
+    sequences = len(RE_IMAGES.findall(body))
     if count:
         scores = [0]
         for image in (1, 2, 3):
@@ -1519,9 +1128,10 @@ def _analyze_images(body, wcount, full=True):
 
 def _analyze_overtagging(body, limit, full=False):
     tags = len(RE_USER_TAGS.findall(body))
-    score = int((tags <= limit))
     if tags > limit:
         score = limit / tags
+    else:
+        score = int((tags <= limit))
 
     if not full:
         return score
@@ -1529,9 +1139,10 @@ def _analyze_overtagging(body, limit, full=False):
 
 
 def _analyze_tags(tags, limit, full=False):
-    score = int((len(tags) <= limit))
     if limit and len(tags) > limit:
         score = limit / len(tags)
+    else:
+        score = int((len(tags) <= limit))
 
     if not full:
         return score
